import logging
from collections import defaultdict

import typer
from typing import Optional
from git import Repo
import matplotlib.pyplot as plt
import pandas as pd

from . import commits
from . import plots

app = typer.Typer()


@app.command()
def count(dir_name: str, year: int = 2023, author_name: str = None, output_csv: Optional[str] = None) -> pd.DataFrame:
    # Count the number of commits each days for git repositories in the folder
    commit_date_counter = defaultdict(int)
    for repo_dir in commits.iter_repos(dir_name):
        repo = Repo(repo_dir)
        if repo.heads != []:
            repo_commit_date_counter = commits.repo_daily_commits(repo, year, author_name)
            for key, count in repo_commit_date_counter.items():
                commit_date_counter[key] += 1
    # Construct a pandas DataFrame with this data
    df = pd.DataFrame({"date": commit_date_counter.keys(), "count": commit_date_counter.values()})
    df["date"] = pd.to_datetime(df["date"])
    
    if output_csv is not None:
        df.to_csv(output_csv, sep=";", index=False)
    return df

@app.command()
def plot(input_csv: str, output_plot: Optional[str] = None):
    df = pd.read_csv(input_csv, sep=";")
    df["date"] = pd.to_datetime(df["date"])
    plots.year_of_contributions(df)
    plt.show()


@app.command()
def pipe(dir: str = ".", author: Optional[str] = None, year: Optional[int] = None, output_csv: Optional[str] = None, output_plot: Optional[str] = None):
    df = count(dir_name=dir, year=year, author_name=author, output_csv=output_csv)
    if output_csv:
        df.to_csv(output_csv, sep=";")
    plots.year_of_contributions(df)
    if output_plot:
        plt.savefig(output_plot)


def main():
    app()


if __name__ == "__main__":
    main()